@@ -6,7 +6,7 @@
  * to grow...
  *  Copyright (C) 1994  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: trace.c,v 1.9 1995/11/16 15:34:05 mike Exp $
+ * $Id: trace.c,v 1.10 1996/07/26 12:03:30 mike Exp $
  * $Source: /usr/CVS/ibcs/Tools/trace.c,v $
  */
 #include <linux/personality.h>
@@ -18,7 +18,40 @@
 #include <ibcs/trace.h>
 
 
-_syscall1(int, personality, int, pers)
+_syscall1(int, personality, int, pers);
+
+
+#ifdef __sparc__
+static int
+set_trace (int val)
+{
+	int res;
+
+	__asm__ volatile ("mov %1,%%o0\n\t"
+			  "mov 193,%%g1\n\t"
+			  "t 8\n\t"
+			  "mov %%o0,%0" : "=r" (res) : "r" (val) : "o0", "g1");
+        return res;
+}
+
+static int
+trace_func_set(int per, int call, int val)
+{
+	int res;
+
+	__asm__ volatile ("mov %1,%%o0\n\t"
+			  "mov %2,%%o1\n\t"
+			  "mov %3,%%o2\n\t"
+			  "mov 194,%%g1\n\t"
+			  "t 8\n\t"
+			  "mov %%o0,%0"
+			  : "=r" (res)
+			  : "r" (per), "r" (call), "r" (val)
+		          : "o0", "o1", "o2", "g1");
+	return res;
+}
+
+#else /* __sparc__ */
 
 static int
 set_trace(int val)
@@ -44,6 +77,7 @@ trace_func_set(int per, int call, int val)
 
 	return res;
 }
+#endif /* __sparc__ */
 
 
 struct t_code {
