@@ -84,27 +84,45 @@ static int parse_comments(struct linux_binprm *exe_bprm, COFF_SCNHDR * sect);
  *  clues might be there. Or use 'strings' if you don't have mcs.
  */
 struct coff_clue {
+    short terminal;	/* non-zero to stop parsing with this entry */
     short len;		/* negative number uses strstr for lookup */
     char *text;
-    unsigned long personality;
+    unsigned long mask_and, mask_or;
 };
 
 static struct coff_clue coff_clues[] = {
     /* Wyse Unix V/386 3.2.1[A]. */
-    { 36, "@(#) UNIX System V/386 Release 3.2.1",	PER_WYSEV386 },
+    { 1, 36, "@(#) UNIX System V/386 Release 3.2.1",
+	0,	PER_WYSEV386 | PER_SHORT_INODE },
 
     /* SCO Unix V 3.2, 3.2.2, 3.2.4, 3.2.4.2 etc. */
-    { 23, "@(#) crt1.s 1.8 89/05/30",			PER_SCOSVR3 },
-    { 16, "@(#)SCO UNIX 3.2",				PER_SCOSVR3 },
-    { 18, "\"@(#) SCO UNIX 3.2",			PER_SCOSVR3 },
-    { 17, "@(#) SCO UNIX 3.2",				PER_SCOSVR3 },
-    { 11, "@(#)SCO 3.2",				PER_SCOSVR3 },
+    { 1, 23, "@(#) crt1.s 1.8 89/05/30",
+	0, 	PER_SCOSVR3 | PER_SHORT_INODE },
+    { 1, 16, "@(#)SCO UNIX 3.2",
+	0, 	PER_SCOSVR3 | PER_SHORT_INODE },
+    { 1, 18, "\"@(#) SCO UNIX 3.2",
+	0, 	PER_SCOSVR3 | PER_SHORT_INODE },
+    { 1, 17, "@(#) SCO UNIX 3.2",	
+	0, 	PER_SCOSVR3 | PER_SHORT_INODE },
+    { 1, 11, "@(#)SCO 3.2",
+	0, 	PER_SCOSVR3 | PER_SHORT_INODE },
+
+    /* SCO Unix 3.2.4.2 OpenServer 5 gives 32 bit inodes except
+     * programs compiled with ods30 compatibilty. In fact OS5
+     * always gives 32 bits but the library drops the top 16 in
+     * odt30 mode. We know what should happen and do it however.
+     */
+   { 0, 32, "@(#) crt1.s.source 20.1 94/12/04",
+	0,	PER_SCOSVR3 },
+   { 1, 13, "ods_30_compat",
+	~0,	PER_SHORT_INODE },
 
     /* Interactive (ISC) 4.0 */
-    { -1, "INTERACTIVE",				PER_ISCR4   },
+    { 1, -1, "INTERACTIVE",
+	0, 	PER_ISCR4   },
 
     /* End of table */
-    { 0, 0, 0 }
+    { 0, 0, 0, 0, 0 }
 };
 
 
@@ -210,20 +228,11 @@ load_object (struct linux_binprm * bprm, struct pt_regs *regs, int lib_ok)
     char not_pageable = 0;	/* Can we demand page the executable?  */
     unsigned long p = bprm->p;
 
-#ifdef COFF_TRACE
-    if (ibcs_trace & TRACE_COFF_LD)
-        printk (KERN_DEBUG "COFF: exec %s\n", bprm->filename);
-#endif
-
 /*
  *  Validate the magic value for the object file.
  */
     do {
 	if (COFF_I386BADMAG (*coff_hdr)) {
-#ifdef COFF_TRACE
-	    if (ibcs_trace & TRACE_COFF_LD)
-	        printk (KERN_DEBUG "COFF: bad filehdr magic\n");
-#endif
 	    status = -ENOEXEC;
 	    break;
 	}
@@ -770,6 +779,12 @@ load_object (struct linux_binprm * bprm, struct pt_regs *regs, int lib_ok)
 			break;
 	        sect_ptr = (COFF_SCNHDR *) &((char *) sect_ptr) [COFF_SCNHSZ];
 	    }
+
+	    /* If no .comments section was found there is no way to
+	     * figure out the personality. Odds on it is SCO though...
+	     */
+	    if (nIndex == sections)
+		current->personality = PER_SCOSVR3;
 	}
 #ifdef INIT_MM
 	if (current->exec_domain && current->exec_domain->use_count)
@@ -811,10 +826,6 @@ load_object (struct linux_binprm * bprm, struct pt_regs *regs, int lib_ok)
 /*
  *  Return the completion status.
  */
-#ifdef COFF_TRACE
-    if (ibcs_trace & TRACE_COFF_LD)
-        printk (KERN_DEBUG "COFF: binfmt_coff: result = %d\n", status);
-#endif
     return (status);
 }
 
@@ -967,7 +978,7 @@ parse_comments (struct linux_binprm *exe_bprm, COFF_SCNHDR * sect)
 {
 	unsigned long offset, nbytes;
 	char *buffer;
-	int status;
+	int status, hits;
 
 /*
  *  Fetch the size of the section. There must be something in there or the
@@ -980,6 +991,7 @@ parse_comments (struct linux_binprm *exe_bprm, COFF_SCNHDR * sect)
 	if (!(buffer = (char *)__get_free_page(GFP_KERNEL)))
 		return 0;
 
+	hits = 0;
 	offset = COFF_LONG(sect->s_scnptr);
 	while (nbytes > 0) {
 		char *p;
@@ -1020,6 +1032,8 @@ parse_comments (struct linux_binprm *exe_bprm, COFF_SCNHDR * sect)
 			for (clue=coff_clues; clue->len; clue++) {
 				if ((clue->len < 0 && strstr(p, clue->text))
 				|| (clue->len > 0 && !strncmp(p, clue->text, clue->len))) {
+					current->personality &= clue->mask_and;
+					current->personality |= clue->mask_or;
 #ifdef COFF_TRACE
 					if (ibcs_trace & TRACE_COFF_LD) {
 						printk(KERN_DEBUG
@@ -1030,12 +1044,14 @@ parse_comments (struct linux_binprm *exe_bprm, COFF_SCNHDR * sect)
 							clue->text);
 						printk(KERN_DEBUG
 							"COFF:  giving 0x%08lx\n",
-							clue->personality);
+							current->personality);
 					}
 #endif
-					current->personality = clue->personality;
-					free_page((unsigned long)buffer);
-					return (1);
+					if (clue->terminal) {
+						free_page((unsigned long)buffer);
+						return (1);
+					}
+					hits++;
 				}
 			}
 			*(buffer + PAGE_SIZE - 1) = c;
@@ -1059,7 +1075,7 @@ parse_comments (struct linux_binprm *exe_bprm, COFF_SCNHDR * sect)
 	}
 
 	free_page((unsigned long)buffer);
-	return (0);
+	return hits;
 }
 
 
