@@ -152,7 +152,7 @@ unsigned long * create_elf_tables(char * p,int argc,int envc,struct elfhdr * exe
 		mpnt->vm_inode = NULL;
 		mpnt->vm_offset = 0;
 		mpnt->vm_ops = NULL;
-		insert_vm_struct(current, mpnt);
+		insert_vm_struct(current->mm, mpnt);
 #ifdef __NR_mlock
 		current->MM(total_vm) += (mpnt->vm_end - mpnt->vm_start) >> PAGE_SHIFT;
 #endif
@@ -220,6 +220,7 @@ static unsigned int load_elf_interp(struct elfhdr * interp_elf_ex,
 	struct elf_phdr *eppnt;
 	unsigned int len;
 	unsigned int load_addr;
+	int load_addr_set = 0;
 	int elf_exec_fileno;
 	int elf_bss;
 #ifndef STACK_TOP
@@ -278,7 +279,7 @@ static unsigned int load_elf_interp(struct elfhdr * interp_elf_ex,
 	    if (eppnt->p_flags & PF_R) elf_prot = PROT_READ;
 	    if (eppnt->p_flags & PF_W) elf_prot |= PROT_WRITE;
 	    if (eppnt->p_flags & PF_X) elf_prot |= PROT_EXEC;
-		if (interp_elf_ex->e_type == ET_EXEC || load_addr != 0) {
+		if (interp_elf_ex->e_type == ET_EXEC || load_addr_set) {
 			elf_type |= MAP_FIXED;
 			vaddr = eppnt->p_vaddr;
 	    }
@@ -298,7 +299,7 @@ static unsigned int load_elf_interp(struct elfhdr * interp_elf_ex,
 #else
 #  ifdef MAP_DENYWRITE
 			    MAP_PRIVATE | MAP_DENYWRITE |
-			    ((interp_elf_ex->e_type == ET_EXEC || load_addr)
+			    ((interp_elf_ex->e_type == ET_EXEC || load_addr_set)
 				? MAP_FIXED : 0),
 #  else
 			    MAP_PRIVATE | (interp_elf_ex->e_type == ET_EXEC ? MAP_FIXED : 0),
@@ -307,8 +308,10 @@ static unsigned int load_elf_interp(struct elfhdr * interp_elf_ex,
 			    eppnt->p_offset & 0xfffff000);
 #endif /* PF_R */
 	    
-	    if(!load_addr && interp_elf_ex->e_type == ET_DYN)
+	    if (!load_addr_set && interp_elf_ex->e_type == ET_DYN) {
 	      load_addr = error;
+	      load_addr_set = 1;
+	    }
 	    k = load_addr + eppnt->p_vaddr + eppnt->p_filesz;
 	    if(k > elf_bss) elf_bss = k;
 	    if (error < 0 && error > -1024) break;  /* Real error */
@@ -406,6 +409,7 @@ static int load_elf_binary(struct linux_binprm * bprm, struct pt_regs * regs)
   	struct exec interp_ex;
 	struct inode *interpreter_inode;
 	unsigned int load_addr;
+	int load_addr_set = 0;
 	unsigned int interpreter_type = INTERPRETER_NONE;
 	unsigned char ibcs2_interpreter;
 	int i;
@@ -776,8 +780,10 @@ static int load_elf_binary(struct linux_binprm * bprm, struct pt_regs * regs)
 				elf_stack = elf_ppnt->p_vaddr & 0xfffff000;
 #endif
 			
-			if(!load_addr) 
+			if (!load_addr_set) {
 			  load_addr = elf_ppnt->p_vaddr - elf_ppnt->p_offset;
+			  load_addr_set = 1;
+			}
 			k = elf_ppnt->p_vaddr;
 			if(k < start_code) start_code = k;
 			k = elf_ppnt->p_vaddr + elf_ppnt->p_filesz;
