@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: binfmt_xout.c,v 1.30 1996/03/29 17:33:30 mike Exp $
+ * $Id: binfmt_xout.c,v 1.32 1997/05/10 14:45:53 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/binfmt_xout.c,v $
  *
  * This file is based upon code written by Al Longyear for the COFF file
@@ -468,9 +468,9 @@ load_object (struct linux_binprm * bprm, struct pt_regs *regs, int lib_ok)
 		current->MM(start_stack) = bprm->p;
 
 #ifdef PER_XENIX
-		current->personality = PER_XENIX;
+		current->personality = PER_XENIX | PER_SHORT_INODE;
 #else
-		current->personality = PER_SVR3;
+		current->personality = PER_SVR3 | PER_SHORT_INODE;
 #endif
 #ifdef INIT_MM
 		if (current->exec_domain && current->exec_domain->use_count)
@@ -567,6 +567,7 @@ seg_again:
 				addr = PAGE_ALIGN(addr + s->xs_vsize);
 				s->xs_rbase = ldt_info.base_addr;
 			}
+#endif
 			bss_size = seglist[i].xs_vsize-seglist[i].xs_psize;
 			bss_base = seglist[i].xs_rbase+seglist[i].xs_psize;
 
@@ -607,7 +608,7 @@ seg_again:
 			if (s->xs_rbase+s->xs_vsize > current->MM(brk))
 				current->MM(start_brk) =
 				current->MM(brk) = PAGE_ALIGN(s->xs_rbase+s->xs_vsize);
-
+#ifdef XOUT_SEGMENTS
 			if (ldt_info.seg_32bit) {
 				ldt_info.limit = (TASK_SIZE-1) >> 12;
 				ldt_info.limit_in_pages = 1;
