@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Mike Jagdis
  *
- * $Id: bsd.c,v 1.12 1995/12/12 10:31:03 mike Exp $
+ * $Id: bsd.c,v 1.13 1996/07/26 12:03:32 mike Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/bsd.c,v $
  */
 
@@ -136,15 +136,16 @@ bsd_fcntl(struct pt_regs *regs)
 	int arg1, arg2, arg3;
 	int error, retval;
 
+#ifndef __sparc__
 	error = verify_area(VERIFY_READ,
 			((unsigned long *)regs->esp)+1,
 			3*sizeof(long));
 	if (error)
 		return error;
-
-	arg1 = get_fs_long (((unsigned long *) regs->esp) + (1));
-	arg2 = get_fs_long (((unsigned long *) regs->esp) + (2));
-	arg3 = get_fs_long (((unsigned long *) regs->esp) + (3));
+#endif
+	arg1 = get_syscall_parameter (regs, 0);
+	arg2 = get_syscall_parameter (regs, 1);
+	arg3 = get_syscall_parameter (regs, 2);
 
 	switch (arg2) {
 		/* These match the Linux commands. */
@@ -170,9 +171,13 @@ bsd_fcntl(struct pt_regs *regs)
 		case 7: /* F_GETLK */
 		case 8: /* F_SETLK */
 		case 9: /* F_SETLKW */
+#ifdef __sparc__
+		        (((struct flock *)arg3)->l_type)--;
+#else
 			__asm__("decw %%fs:%0"
 				: /* no outputs */
 				: "m" (((struct flock *)arg3)->l_type));
+#endif
 #ifdef IBCS_TRACE
 			if ((ibcs_trace & TRACE_API) || ibcs_func_p->trace) {
 				struct flock fl;
@@ -189,9 +194,13 @@ bsd_fcntl(struct pt_regs *regs)
 #endif
 			/* okay do some magic here */
 			retval = SYS(fcntl)(arg1, arg2-2, arg3);
+#ifdef __sparc__
+			(((struct flock *)arg3)->l_type)++;
+#else
 			__asm__("incw %%fs:%0"
 				: /* no outputs */
 				: "m" (((struct flock *)arg3)->l_type));
+#endif
 			return retval;
 
 		default:
