@@ -6,7 +6,7 @@
  *  Written by Drew Sullivan.
  *  Rewritten by Mike Jagdis.
  *
- * $Id: ioctl.c,v 1.47 1996/06/06 16:23:02 mike Exp $
+ * $Id: ioctl.c,v 1.51 1997/05/10 14:45:53 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/ioctl.c,v $
  */
 #include <linux/config.h>
@@ -165,6 +165,13 @@ do_ioctl(struct pt_regs *regs, int fd, unsigned long ioctl_num, void *arg)
 		 */
 		case 'E'|0x80:
 			return -EINVAL;
+
+		/* SCO channel mapping. I can't find any documentation
+		 * for this. These are the LD?MAP ioctls defined in
+		 * sys/termio.h and sys/emap.h. They are used by mapchan.
+		 */
+		case 'D':
+			return -EINVAL;
 	}
 
 	/* If we haven't handled it yet it must be a BSD style ioctl
@@ -923,6 +930,7 @@ static int ibcs_ioctl_stream(struct pt_regs *regs,
 			return 0;
 		}
 
+		/* FIXME: These are bogus. */
 		case 011: /* I_SETSIG */
 			return SYS(ioctl)(fd, FIOSETOWN, current->pid);
 		case 012: /* I_GETSIG */
@@ -982,6 +990,18 @@ static int ibcs_ioctl_stream(struct pt_regs *regs,
 		case 023: /* I_SWROPT */
 		case 040: /* I_SETCLTIME */
 			return 0; /* Lie... */
+		case 042: /* I_CANPUT */
+			/* Arg is the priority band in question. We only
+			 * support one priority band so arg must be 0.
+			 * If the band is writable we should return 1, if
+			 * the band is flow controlled we should return 0.
+			 */
+			if (arg)
+				return -EINVAL;
+
+			/* FIXME: How can we test if a write would block? */
+			return 1;
+
 		case 024: /* I_GWROPT */
 		case 025: /* I_LIST */
 		case 026: /* I_PLINK */
@@ -995,7 +1015,6 @@ static int ibcs_ioctl_stream(struct pt_regs *regs,
 		case 036: /* I_GETBAND */
 		case 037: /* I_ATMARK */
 		case 041: /* I_GETCLTIME */
-		case 042: /* I_CANPUT */
 #endif
 			/* Unsupported - drop out. */
 	}
@@ -1059,6 +1078,11 @@ static int sco_ioctl_tape(int fd, unsigned int func, unsigned long arg)
 			mtop.mt_op = MTEOM;
 			break;
 
+		case 24:  /* MT_SETBLK */
+			mtop.mt_op = MTSETBLK;
+			mtop.mt_count = arg;
+			break;
+
 		case 25:  /* MT_LOCK */
 			mtop.mt_op = MTLOCK;
 			break;
@@ -1084,7 +1108,6 @@ static int sco_ioctl_tape(int fd, unsigned int func, unsigned long arg)
 		case 17:  /* MT_PUTVTBL */
 		case 18:  /* MT_SERVO */
 		case 22:  /* MT_FORMPART */
-		case 24:  /* MT_SETBLK */
 		case 38:  /* MT_SETANSI */
 		case 64:  /* MT_REPORT */
 #endif
@@ -1148,6 +1171,11 @@ static int ibcs_ioctl_tape(int fd, unsigned int func, unsigned long arg)
 			mtop.mt_op = MTOFFL;
 			break;
 
+		case 15:	/* T_WRBLKLEN */
+			mtop.mt_op = MTLOCK;
+			mtop.mt_count = arg;
+			break;
+
 		case 16:	/* T_PREVMV */
 			mtop.mt_op = MTLOCK;
 			break;
@@ -1179,7 +1207,6 @@ static int ibcs_ioctl_tape(int fd, unsigned int func, unsigned long arg)
 #if 0
 		case 6:		/* T_STATUS */
 		case 14:	/* T_RDBLKLEN */
-		case 15:	/* T_WRBLKLEN */
 		case 18:	/* T_SBB */
 		case 19:	/* T_SFB */
 		case 23:	/* T_STS */
@@ -1206,8 +1233,8 @@ ibcs_ioctl(struct pt_regs *regs)
 	unsigned int ioctl_num;
 	void *arg;
 
-	fd = (int)get_fs_long(((unsigned long *)regs->esp) + 1);
-	ioctl_num = (unsigned int)get_fs_long(((unsigned long *)regs->esp) + 2);
-	arg = (void *)get_fs_long(((unsigned long *)regs->esp) + 3);
+	fd = (int)get_syscall_parameter (regs, 0);
+	ioctl_num = (unsigned int)get_syscall_parameter (regs, 1);
+	arg = (void *)get_syscall_parameter (regs, 2);
 	return do_ioctl(regs, fd, ioctl_num, arg);
 }
