@@ -4,7 +4,7 @@
  *  Copyright (C) 1993  Joe Portman (baron@hebron.connected.com)
  *  Copyright (C) 1993, 1994  Drew Sullivan (re-worked for iBCS2)
  *
- * $Id: open.c,v 1.28 1996/06/06 16:23:02 mike Exp $
+ * $Id: open.c,v 1.36 1997/05/13 19:56:29 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/open.c,v $
  */
 
@@ -33,6 +33,9 @@
 #include <linux/tty.h>
 #include <linux/time.h>
 #include <linux/malloc.h>
+#include <linux/un.h>
+
+#include <asm/bitops.h>
 
 #include <ibcs/ibcs.h>
 
@@ -238,9 +241,78 @@ int ibcs_mkdir(const char *fname, int mode)
 }
 
 
-int ibcs_open(const char * fname, int flag, int mode)
+int ibcs_mknod(const char *fname, int mode, int dev)
 {
+	/* Linux doesn't allow us to create a directory with mknod(). */
+	if ((mode & 0017000) == 0040000)
+		return ibcs_mkdir(fname, mode);
+	return SYS(mknod)(fname, mode, dev);
+}
+
+
+int ibcs_open(const char *fname, int flag, int mode)
+{
+#ifdef __sparc__
 	return SYS(open)(fname, map_flags(flag, fl_ibcs_to_linux), mode);
+#else
+	int error, fd, old_fs, args[3];
+	char *p;
+	struct sockaddr_un addr;
+
+	fd = SYS(open)(fname, map_flags(flag, fl_ibcs_to_linux), mode);
+	if (fd < 0)
+		return fd;
+
+	/* Sometimes a program may open a pathname which it expects
+	 * to be a named pipe (or STREAMS named pipe) when the
+	 * Linux domain equivalent is a Unix domain socket. (e.g.
+	 * UnixWare uses a STREAMS named pipe /dev/X/Nserver.0 for
+	 * X :0 but Linux uses a Unix domain socket /tmp/.X11-unix/X0)
+	 * It isn't enough just to make the symlink because you cannot
+	 * open() a socket and read/write it. If we spot the error we can
+	 * switch to socket(), connect() and things will likely work
+	 * as expected however.
+	 */
+	if (!S_ISSOCK(current->FD[fd]->f_inode->i_mode))
+		return fd;
+
+	SYS(close)(fd);
+	args[0] = AF_UNIX;
+	args[1] = SOCK_STREAM;
+	args[2] = 0;
+	old_fs = get_fs();
+	set_fs(get_ds());
+	fd = SYS(socketcall)(SYS_SOCKET, args);
+	set_fs(old_fs);
+	if (fd < 0)
+		return fd;
+
+	if ((error = getname(fname, &p))) {
+		SYS(close)(fd);
+		return error;
+	}
+	if (strlen(p) >= UNIX_PATH_MAX) {
+		putname(p);
+		SYS(close)(fd);
+		return -E2BIG;
+	}
+	addr.sun_family = AF_UNIX;
+	strcpy(addr.sun_path, p);
+	putname(p);
+
+	args[0] = fd;
+	args[1] = (int)&addr;
+	args[2] = sizeof(struct sockaddr_un);
+	set_fs(get_ds());
+	error = SYS(socketcall)(SYS_CONNECT, args);
+	set_fs(old_fs);
+	if (error) {
+		SYS(close)(fd);
+		return error;
+	}
+
+	return fd;
+#endif
 }
 
 
@@ -305,11 +377,24 @@ int ibcs_getdents(int fd, char *buf, int nbytes)
 			+ sizeof(unsigned short) + d->d_reclen + 1
 			+ 3) & (~3);
 		if (posn + reclen <= nbytes) {
-			/* According to SCO d_ino is only long in iBCS2
-			 * mode. Anything else should treat it as a short.
-			 * This means non-iBCS2 compiled code will only
-			 * see the low word which is what we want here.
-			 */
+			if (current->personality & PER_SHORT_INODE) {
+				/* read() on a directory only handles
+				 * short inodes but cannot use 0 as that
+				 * indicates an empty directory slot.
+				 * Therefore stat() must also fold
+				 * inode numbers avoiding 0. Which in
+				 * turn means that getdents() must fold
+				 * inodes avoiding 0 - if the program
+				 * was built in a short inode environment.
+				 * If we have short inodes in the dirent
+				 * we also have a two byte pad so we
+				 * can let the high word fall in the pad.
+				 * This makes it a little more robust if
+				 * we guessed the inode size wrong.
+				 */
+				if (!((unsigned long)d->d_ino & 0xffff))
+					d->d_ino = 0xfffffffe;
+			}
 			d->d_reclen = reclen;
 			d->d_off = file->f_pos;
 			memcpy_tofs(buf+posn, d, reclen);
@@ -345,15 +430,16 @@ ibcs_fcntl(struct pt_regs *regs)
 	int arg1, arg2, arg3;
 	int error, retval;
 
+#ifndef __sparc__
 	error = verify_area(VERIFY_READ,
 			((unsigned long *)regs->esp)+1,
 			3*sizeof(long));
 	if (error)
 		return error;
-
-	arg1 = get_fs_long (((unsigned long *) regs->esp) + (1));
-	arg2 = get_fs_long (((unsigned long *) regs->esp) + (2));
-	arg3 = get_fs_long (((unsigned long *) regs->esp) + (3));
+#endif /* __sparc__ */
+	arg1 = get_syscall_parameter (regs, 0);
+	arg2 = get_syscall_parameter (regs, 1);
+	arg3 = get_syscall_parameter (regs, 2);
 
 	switch (arg2) {
 		/* These match the Linux commands. */
@@ -377,9 +463,13 @@ ibcs_fcntl(struct pt_regs *regs)
 		case 5: /* F_GETLK */
 		case 6: /* F_SETLK */
 		case 7: /* F_SETLKW */
+#ifdef __sparc__
+		        (((struct ibcs_flock *)arg3)->l_type)--;
+#else /* __sparc__ */
 			__asm__("decw %%fs:%0"
 				: /* no outputs */
 				: "m" (((struct ibcs_flock *)arg3)->l_type));
+#endif /* __sparc__ */
 #ifdef IBCS_TRACE
 			if ((ibcs_trace & TRACE_API) || ibcs_func_p->trace) {
 				struct ibcs_flock fl;
@@ -397,10 +487,13 @@ ibcs_fcntl(struct pt_regs *regs)
 #endif
 			/* okay do some magic here */
 			retval = SYS(fcntl)(arg1, arg2, arg3);
+#ifdef __sparc__
+			(((struct ibcs_flock *)arg3)->l_type)++;
+#else /* __sparc__ */
 			__asm__("incw %%fs:%0"
 				: /* no outputs */
 				: "m" (((struct ibcs_flock *)arg3)->l_type));
-
+#endif /* __sparc__ */
 			/* Linux doesn't have the sysid field so we need to
 			 * roll the pid down a slot.
 			 */
@@ -424,11 +517,24 @@ ibcs_fcntl(struct pt_regs *regs)
 		 */
 		case 0x6000: /* F_CHSIZE */
 			return SYS(ftruncate)(arg1, arg3);
+#ifndef __sparc__
 		case 0x6001: /* F_RDCHK */
 			return xnx_rdchk(arg1);
+#endif /* __sparc__ */
 
+#ifdef EMU_SCO
+		/* This could be SCO's get highest fd open if the fd we
+		 * are called on is -1 otherwise it could be F_CHKFL.
+		 */
+		case  8: /* F_GETHFDO */
+			if (arg1 == -1)
+				return find_first_zero_bit(&current->files->open_fds,
+							NR_OPEN);
+			/* else fall through to fail */
+#else
 		/* The following are defined but reserved and unknown. */
 		case  8: /* F_CHKFL */
+#endif
 
 		/* These are made from the Xenix locking() system call.
 		 * According to available documentation these would
@@ -451,6 +557,4 @@ ibcs_fcntl(struct pt_regs *regs)
 			return -EINVAL;
 			break;
 	}
-			
-			
 }
