@@ -3,7 +3,7 @@
  *
  *  Copyright 1994-1996  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: socksys.c,v 1.45 1996/03/29 17:33:35 mike Exp $
+ * $Id: socksys.c,v 1.50 1997/05/10 14:45:54 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/socksys.c,v $
  */
 
@@ -151,7 +151,8 @@ socksys_syscall(int fd, int *sp)
 			"recvfrom", "send", "sendto", "setsockopt", "shutdown",
 			"socket", "select", "getipdomain", "setipdomain",
 			"adjtime", "setreuid", "setregid", "gettimeofday",
-			"settimeofday", "getitimer", "setitimer"
+			"settimeofday", "getitimer", "setitimer",
+			"recvmsg", "sendmsg", "sockpair"
 		};
 
 		a0 = (unsigned long)get_fs_long(sp+0);
@@ -162,7 +163,9 @@ socksys_syscall(int fd, int *sp)
 		a5 = (unsigned long)get_fs_long(sp+5);
 		printk(KERN_DEBUG "socksys: [%d] %d: %s (%d) <0x%lx,0x%lx,0x%lx,0x%lx,0x%lx,0x%lx>\n",
 			current->pid, fd,
-			cmd_map[cmd], cmd, a0, a1, a2, a3, a4, a5);
+			(cmd >= 0 && cmd < sizeof(cmd_map)/sizeof(cmd_map[0]))
+				? cmd_map[cmd] : "???",
+			cmd, a0, a1, a2, a3, a4, a5);
 	}
 #endif
 
@@ -237,14 +240,26 @@ socksys_syscall(int fd, int *sp)
 			return ibcs_getsockopt((unsigned long *)sp);
 		case SSYS_SO_LISTEN:
 			return SYS(socketcall)(SYS_LISTEN, sp);
-		case SSYS_SO_RECV:
-			return SYS(socketcall)(SYS_RECV, sp);
-		case SSYS_SO_RECVFROM:
-			return SYS(socketcall)(SYS_RECVFROM, sp);
-		case SSYS_SO_SEND:
-			return SYS(socketcall)(SYS_SEND, sp);
-		case SSYS_SO_SENDTO:
-			return SYS(socketcall)(SYS_SENDTO, sp);
+		case SSYS_SO_RECV: {
+			int err = SYS(socketcall)(SYS_RECV, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
+		case SSYS_SO_RECVFROM: {
+			int err = SYS(socketcall)(SYS_RECVFROM, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
+		case SSYS_SO_SEND: {
+			int err = SYS(socketcall)(SYS_SEND, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
+		case SSYS_SO_SENDTO: {
+			int err = SYS(socketcall)(SYS_SENDTO, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
 		case SSYS_SO_SETSOCKOPT:
 			return ibcs_setsockopt((unsigned long *)sp);
 		case SSYS_SO_SHUTDOWN:
@@ -362,6 +377,73 @@ socksys_syscall(int fd, int *sp)
 
 		case SSYS_SO_ADJTIME:
 			return -EINVAL;
+
+		/* These appear in SCO 3.2v5. I assume that the format of
+		 * a msghdr is identical with Linux. I have not checked.
+		 */
+		case SSYS_SO_RECVMSG: {
+			int err = SYS(socketcall)(SYS_RECVMSG, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
+		case SSYS_SO_SENDMSG: {
+			int err = SYS(socketcall)(SYS_SENDMSG, sp);
+			if (err == -EAGAIN) err = -EWOULDBLOCK;
+			return err;
+		}
+
+		case SSYS_SO_SOCKPAIR: {
+			/* Get a socketpair but replace the socket file
+			 * operations with our own so we can do the
+			 * right thing for ioctls.
+			 */
+			struct inode *ino;
+			int error, args[4], pairin[2], pairout[2];
+			int old_fs = get_fs();
+
+			/* The first two arguments are file descriptors
+			 * of sockets which have already been opened
+			 * and should now be connected back to back.
+			 */
+			if ((error = verify_area(VERIFY_READ, sp, 2*sizeof(long))))
+				return error;
+			pairin[0] = (int)get_fs_long(sp+0);
+			pairin[1] = (int)get_fs_long(sp+1);
+
+			if (pairin[0] >= NR_OPEN
+			|| !current->FD[pairin[0]]
+			|| !(ino = current->FD[pairin[0]]->f_inode)
+			|| !ino->i_sock)
+				return -EBADF;
+
+			args[0] = AF_UNIX;
+			args[1] = ino->u.socket_i.type;
+			args[2] = 0;
+			args[3] = (int)pairout;
+
+			SYS(close)(pairin[0]);
+			SYS(close)(pairin[1]);
+			set_fs(get_ds());
+			error = SYS(socketcall)(SYS_SOCKETPAIR, args);
+			set_fs(old_fs);
+			if (error < 0)
+				return error;
+
+			if (pairout[0] != pairin[0]) {
+				SYS(dup2)(pairout[0], pairin[0]);
+				SYS(close)(pairout[0]);
+			}
+			if (pairout[1] != pairin[1]) {
+				SYS(dup2)(pairout[1], pairin[1]);
+				SYS(close)(pairout[1]);
+			}
+
+			inherit_socksys_funcs(current->FD[pairin[0]], pairin[0]);
+			inherit_socksys_funcs(current->FD[pairin[1]], pairin[1]);
+			MOD_INC_USE_COUNT;
+			MOD_INC_USE_COUNT;
+			return 0;
+		}
 	}
 
 	return -EINVAL;
@@ -647,7 +729,7 @@ ibcs_ioctl_socksys(int fd, unsigned int cmd, void *arg)
 
 
 static int
-spx_write(struct inode *ino, struct file *filep, const char *buf, int count)
+spx_connect(struct inode *ino, struct file *filep, int spxnum)
 {
 	int newfd, err, args[3];
 	struct sockaddr_un addr = {
@@ -655,12 +737,10 @@ spx_write(struct inode *ino, struct file *filep, const char *buf, int count)
 	};
 	int old_fs = get_fs();
 
-	newfd = get_fs_byte(buf);
-
 #ifdef IBCS_TRACE
 	if (ibcs_trace & TRACE_SOCKSYS)
 		printk(KERN_DEBUG "SPX: [%d] %lx choose service %d\n",
-			current->pid, (unsigned long)filep, newfd);
+			current->pid, (unsigned long)filep, spxnum);
 #endif
 
 	/* Rather than use an explicit path to the X :0 server
@@ -676,7 +756,7 @@ spx_write(struct inode *ino, struct file *filep, const char *buf, int count)
 	 *   Hey, this is what we do here now!
 	 */
 	addr.sun_family = AF_UNIX;
-	sprintf(addr.sun_path, "/dev/spx.map/%d", newfd);
+	sprintf(addr.sun_path, "/dev/spx.map/%d", spxnum);
 	set_fs(get_ds());
 	err = SYS(readlink)(addr.sun_path, addr.sun_path,
 				sizeof(addr.sun_path)-1);
@@ -732,17 +812,34 @@ spx_write(struct inode *ino, struct file *filep, const char *buf, int count)
 			current->pid, (unsigned long)filep, (unsigned long)ino,
 			(unsigned long)current->FD[newfd]->f_inode);
 #endif
+
 	inherit_socksys_funcs(filep, newfd);
-	current->FD[newfd]->f_inode->i_rdev = ino->i_rdev;
-	iput(ino);
 	filep->f_inode = current->FD[newfd]->f_inode;
-	if (--current->FD[newfd]->f_count == 0)
-		current->FD[newfd] = NULL;
+	filep->f_inode->i_rdev = ino->i_rdev;
+	filep->f_inode->i_flock = ino->i_flock;
+	filep->f_inode->u.socket_i.file = filep;
+	iput(ino);
+	FD_CLR(newfd, &current->files->close_on_exec);
+	FD_CLR(newfd, &current->files->open_fds);
+	current->FD[newfd]->f_count--;
+	current->FD[newfd] = NULL;
 
 	return 1;
 }
 
 
+static int
+spx_write(struct inode *ino, struct file *filep, const char *buf, int count)
+{
+	int error;
+
+	if ((error = verify_area(VERIFY_READ, buf, 1*sizeof(char))))
+		return error;
+
+	return spx_connect(ino, filep, get_fs_byte(buf));
+}
+
+
 static int
 socksys_open(struct inode *ino, struct file *filep)
 {
@@ -771,7 +868,19 @@ socksys_open(struct inode *ino, struct file *filep)
 	 */
 	if (MINOR(ino->i_rdev) == 1) {
 		filep->f_op = &spx_fops;
+#if 0
+		/* It seems that there was an early spx implementation
+		 * that only supported one destination and which
+		 * required nothing but an open() to connect. When we
+		 * find out how to identify such cases we can wrap this
+		 * code in a conditional and enable it. In the meantime
+		 * you might find "ln -s /tmp/.X11-unix/X0 /dev/spx"
+		 * will work?
+		 */
+		return spx_connect(ino, filep, 1);
+#else
 		return 0;
+#endif
 	}
 
 	/*
@@ -786,7 +895,7 @@ socksys_open(struct inode *ino, struct file *filep)
 #ifdef EMU_XTI
 	struct T_private *priv;
 #endif
-	int fd, err, args[3];
+	int fd, args[3];
 	int old_fs = get_fs();
 
 	/* Grab a socket. */
@@ -849,7 +958,7 @@ socksys_open(struct inode *ino, struct file *filep)
 	if (!priv) {
 		SYS(close)(fd);
 		MOD_DEC_USE_COUNT;
-		return err;
+		return -ENOMEM;
 	}
 	priv->state = TS_UNBND;
 	priv->offset = 0;
@@ -866,14 +975,18 @@ socksys_open(struct inode *ino, struct file *filep)
 			current->pid, (unsigned long)filep);
 #endif
 	inherit_socksys_funcs(filep, fd);
-	current->FD[fd]->f_inode->i_rdev = ino->i_rdev;
-	iput(ino);
 	filep->f_inode = current->FD[fd]->f_inode;
+	filep->f_inode->i_rdev = ino->i_rdev;
+	filep->f_inode->i_flock = ino->i_flock;
+	filep->f_inode->u.socket_i.file = filep;
+	iput(ino);
 #ifdef EMU_XTI
 	filep->private_data = priv;
 #endif
-	if (--current->FD[fd]->f_count == 0)
-		current->FD[fd] = NULL;
+	FD_CLR(fd, &current->files->close_on_exec);
+	FD_CLR(fd, &current->files->open_fds);
+	current->FD[fd]->f_count--;
+	current->FD[fd] = NULL;
 
 	return 0;
 } /* XTI */
