@@ -6,7 +6,7 @@
  *  Hacked by Eric Youngdale for iBCS.
  *  Added to by Drew Sullivan.
  *
- * $Id: stat.c,v 1.9 1995/12/12 10:31:07 mike Exp $
+ * $Id: stat.c,v 1.12 1997/05/12 23:13:40 jaggy Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/stat.c,v $
  */
 #define __KERNEL__ 1
@@ -29,6 +29,32 @@
 #include <linux/mm.h>
 
 #include <ibcs/ibcs.h>
+
+
+#ifdef __sparc__
+static void cp_ibcs_stat(struct inode * inode, struct ibcs_stat * statbuf)
+{
+	struct ibcs_stat tmp;
+
+	memset ((void *) &tmp, 0, sizeof (tmp));
+	tmp.st_dev = inode->i_dev;
+	tmp.st_ino = inode->i_ino;
+	tmp.st_mode = inode->i_mode;
+	tmp.st_nlink = inode->i_nlink;
+	tmp.st_uid = inode->i_uid;
+	tmp.st_gid = inode->i_gid;
+	tmp.st_rdev = inode->i_rdev;
+	tmp.st_size = inode->i_size;
+	tmp.st_atime.tv_sec = inode->i_atime;
+	tmp.st_mtime.tv_sec = inode->i_mtime;
+	tmp.st_ctime.tv_sec = inode->i_ctime;
+	tmp.st_blksize = inode->i_blksize;
+	tmp.st_blocks  = inode->i_blocks;
+	memcpy_tofs(statbuf,&tmp,sizeof(tmp));
+}
+
+#else /* if not sparc... */
+
 /*
  * Believe it or not, the original stat structure is compatible with ibcs2.
  * The xstat struct used by SVr4 is different than our new struct, but we will
@@ -39,15 +65,17 @@ static void cp_ibcs_stat(struct inode * inode, struct ibcs_stat * statbuf)
 	struct ibcs_stat tmp;
 
 	/* Note that we have to fold a long inode number down to a short.
-	 * This must match what happens in open.c:ibcs_getdents() since code
-	 * that figures out cwd needs the inodes to match. I don't know
-	 * if it should match coff.c:ibcs_read() on a directory. From
-	 * tests it seems that that always sets a large inode number to
-	 * 0xfffe. In theory this would break old pwd routines but SCO
-	 * seems to do it. Many getcwd() library function just do a
-	 * piped "pwd" anyway...
+	 * This must match what happens in coff:ibcs_read() and
+	 * open.c:ibcs_getdents() since code that figures out cwd needs
+	 * the inodes to match. Because it must match read() on a
+	 * directory we have to avoid the situation where we end up
+	 * with a zero inode value. A zero inode value in a read()
+	 * on a directory indicates an empty directory slot.
 	 */
-	tmp.st_ino = (unsigned long)inode->i_ino & 0x0000ffff;
+	if ((unsigned long)inode->i_ino & 0xffff)
+		tmp.st_ino = (unsigned long)inode->i_ino & 0xffff;
+	else
+		tmp.st_ino = 0xfffe;
 
 	tmp.st_dev = inode->i_dev;
 	tmp.st_mode = inode->i_mode;
@@ -61,7 +89,7 @@ static void cp_ibcs_stat(struct inode * inode, struct ibcs_stat * statbuf)
 	tmp.st_ctime = inode->i_ctime;
 	memcpy_tofs(statbuf,&tmp,sizeof(tmp));
 }
-
+#endif /* not sparc */
 
 #ifdef __cplusplus
 extern "C" 
