@@ -3,7 +3,7 @@
  *
  *  Copyright 1994, 1995  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: stream.c,v 1.8 1995/12/14 10:46:07 mike Exp $
+ * $Id: stream.c,v 1.9 1996/07/26 12:03:39 mike Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/stream.c,v $
  */
 
@@ -40,7 +40,7 @@ ibcs_getmsg(struct pt_regs *regs)
 	int fd;
 	struct inode *ino;
 
-	fd = (int)get_fs_long(((unsigned long *)regs->esp) + 1);
+	fd = (int)get_syscall_parameter (regs, 0);
 
 	if (!current->FD[fd]
 	|| !(ino = current->FD[fd]->f_inode))
@@ -63,7 +63,7 @@ ibcs_putmsg(struct pt_regs *regs)
 	int fd;
 	struct inode *ino;
 
-	fd = (int)get_fs_long(((unsigned long *)regs->esp) + 1);
+	fd = (int)get_syscall_parameter (regs, 0);
 
 	if (!current->FD[fd]
 	|| !(ino = current->FD[fd]->f_inode))
@@ -88,7 +88,7 @@ ibcs_getpmsg(struct pt_regs *regs)
 	int fd;
 	struct inode *ino;
 
-	fd = (int)get_fs_long(((unsigned long *)regs->esp) + 1);
+	fd = (int)get_syscall_parameter (regs, 0);
 
 	if (!current->FD[fd]
 	|| !(ino = current->FD[fd]->f_inode))
@@ -107,7 +107,7 @@ ibcs_putpmsg(struct pt_regs *regs)
 	int fd;
 	struct inode *ino;
 
-	fd = (int)get_fs_long(((unsigned long *)regs->esp) + 1);
+	fd = (int)get_syscall_parameter (regs, 0);
 
 	if (!current->FD[fd]
 	|| !(ino = current->FD[fd]->f_inode))
