@@ -3,7 +3,7 @@
  *
  *  Copyright 1995, 1996  Mike Jagdis (jaggy@purplet.demon.co.uk)
  *
- * $Id: timod.c,v 1.20 1996/05/31 10:39:27 mike Exp $
+ * $Id: timod.c,v 1.21 1996/07/26 12:03:40 mike Exp $
  * $Source: /usr/CVS/ibcs/iBCSemul/timod.c,v $
  */
 
@@ -39,6 +39,14 @@
 #include <ibcs/trace.h>
 #endif
 
+#ifdef __sparc__
+/* ibcs2 wants to access the stack pointer, let's give it an alias
+ * for the sparc
+ */
+#define esp u_regs[UREG_FP]
+#endif
+
+extern void kill_fasync(struct fasync_struct *fa, int sig);
 
 #ifdef IBCS_TRACE
 static char *
@@ -1329,10 +1337,58 @@ ibcs_ioctl_sockmod(int fd, unsigned int func, void *arg)
 		return TBADF;
 
 	switch (func) {
+#ifdef __sparc__
+            	case 110: { /* SI_GETUDATA -- Solaris */
+			struct {
+				int tidusize, addrsize, optsize, etsdusize;
+				int servtype, so_state, so_options;
+			        int tsdusize;
+			    
+			        /* Socket parameters */
+			        int family, type, protocol;
+			} *it = arg;
+
+#ifdef IBCS_TRACE
+			if (ibcs_trace & TRACE_STREAMS)
+				printk(KERN_DEBUG "iBCS: [%d] %lx new_getudata\n",
+					current->pid,
+					(unsigned long)current->FD[fd]);
+#endif
+			error = verify_area(VERIFY_WRITE, it, sizeof(*it));
+			if (error)
+				return (-error << 8) | TSYSERR;
+
+			put_fs_long(16384, &it->tidusize);
+			put_fs_long(sizeof(struct sockaddr), &it->addrsize);
+			put_fs_long(-1, &it->optsize);
+			put_fs_long(0, &it->so_state);
+			put_fs_long(0, &it->so_options);
+			put_fs_long(16384, &it->tsdusize);
+			
+			switch (ino->u.socket_i.type) {
+				case SOCK_STREAM:
+					put_fs_long(1, &it->etsdusize);
+					put_fs_long(2, &it->servtype);
+					break;
+				default:
+					put_fs_long(-2, &it->etsdusize);
+					put_fs_long(3, &it->servtype);
+					break;
+			}
+			put_fs_long (ino->u.socket_i.ops->family, &it->family);
+			put_fs_long (ino->u.socket_i.type, &it->type);
+			put_fs_long (ino->u.socket_i.ops->family, &it->protocol);
+			return 0;
+		}
+		    
+#endif
 		case 101: { /* SI_GETUDATA */
 			struct {
 				int tidusize, addrsize, optsize, etsdusize;
 				int servtype, so_state, so_options;
+#ifdef __sparc__
+			        int tsdusize;
+#endif
 			} *it = arg;
 
 #ifdef IBCS_TRACE
@@ -1351,6 +1407,9 @@ ibcs_ioctl_sockmod(int fd, unsigned int func, void *arg)
 			put_fs_long(0, &it->so_state);
 			put_fs_long(0, &it->so_options);
 
+#ifdef __sparc__
+			put_fs_long (16384, &it->tsdusize);
+#endif
 			switch (ino->u.socket_i.type) {
 				case SOCK_STREAM:
 					put_fs_long(1, &it->etsdusize);
@@ -1394,13 +1453,13 @@ timod_getmsg(int fd, struct inode *ino, int is_pmsg, struct pt_regs *regs)
 	int error;
 	struct strbuf ctl, dat;
 
-	ctlptr = (struct strbuf *)get_fs_long(((unsigned long *)regs->esp) + 2);
-	datptr = (struct strbuf *)get_fs_long(((unsigned long *)regs->esp) + 3);
+	ctlptr = get_syscall_parameter (regs, 1);
+	datptr = get_syscall_parameter (regs, 2);
 	if (!is_pmsg) {
-		flags_p = (int *)get_fs_long(((unsigned long *)regs->esp) + 4);
+		flags_p = (int *)get_syscall_parameter (regs, 3);
 	} else {
-		band_p = (int *)get_fs_long(((unsigned long *)regs->esp) + 4);
-		flags_p = (int *)get_fs_long(((unsigned long *)regs->esp) + 5);
+		band_p = (int *)get_syscall_parameter (regs, 3);
+		flags_p = (int *)get_syscall_parameter (regs, 4);
 		error = verify_area(VERIFY_WRITE, band_p, sizeof(int));
 		if (error)
 			return error;
@@ -1481,13 +1540,13 @@ timod_putmsg(int fd, struct inode *ino, int is_pmsg, struct pt_regs *regs)
 	int flags, error, band;
 	struct strbuf ctl, dat;
 
-	ctlptr = (struct strbuf *)get_fs_long(((unsigned long *)regs->esp) + 2);
-	datptr = (struct strbuf *)get_fs_long(((unsigned long *)regs->esp) + 3);
+	ctlptr = (struct strbuf *)get_syscall_parameter (regs, 1);
+	datptr = (struct strbuf *)get_syscall_parameter (regs, 2);
 	if (!is_pmsg) {
-		flags = (int)get_fs_long(((unsigned long *)regs->esp) + 4);
+		flags = (int)get_syscall_parameter (regs, 3);
 	} else {
-		band = (int)get_fs_long(((unsigned long *)regs->esp) + 4);
-		flags = (int)get_fs_long(((unsigned long *)regs->esp) + 5);
+		band = (int)get_syscall_parameter (regs, 3);
+		flags = (int)get_syscall_parameter (regs, 4);
 	}
 
 	if (ctlptr) {
