@@ -1,8 +1,8 @@
 /*
  * ibcs trace -- mask of trace values
  *
- * $Id: trace.h,v 1.8 1995/04/12 10:12:44 mike Exp $
- * $Source: /u3/CVS/ibcs/include/ibcs/trace.h,v $
+ * $Id: trace.h,v 1.9 1995/11/16 15:34:23 mike Exp $
+ * $Source: /usr/CVS/ibcs/include/ibcs/trace.h,v $
  */
 #define	TRACE_API	0x00000001 /* all call/return values */
 #define	TRACE_IOCTL	0x00000002 /* all ioctl calls */
@@ -11,9 +11,10 @@
 #define	TRACE_SIGNAL_F	0x00000010 /* signal calls that fail */
 #define	TRACE_SOCKSYS	0x00000020 /* socksys and spx devices */
 #define	TRACE_COFF_LD	0x00000040 /* COFF loader */
-#define	TRACE_XOUT_LD	0x00000080 /* XOUT loader */
-#define	TRACE_XOUT_DB	0x00000100 /* XOUT loader blocks before launch */
-#define	TRACE_STREAMS	0x00000200 /* STREAMS faking */
+#define	TRACE_ELF_LD	0x00000080 /* ELF loader */
+#define	TRACE_XOUT_LD	0x00000100 /* XOUT loader */
+#define	TRACE_XOUT_DB	0x00000200 /* XOUT loader blocks before launch */
+#define	TRACE_STREAMS	0x00000400 /* STREAMS faking */
 #define	TRACE_FUNC	0x10000000 /* trace this function */
 
 extern int ibcs_trace;
